﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Experimental.U2D.Animation;

namespace CharacterGenerator
{
    public class CharacterGraphics : MonoBehaviour
    {
        [NonSerialized] public string[] labels;

        public Action<string, int> OnAllPartsSwitched;

        [SerializeField] private SpriteResolver[] spriteResolvers;

        private Dictionary<string, SpriteResolver> spriteResolverByCategory;

        public void SwitchParts(string[] categories, string label)
        {
            foreach (var category in categories)
            {
                SpriteResolver spriteResolver = spriteResolverByCategory[category];

                spriteResolver.SetCategoryAndLabel(category, label);
            }
        }

        public void SwitchParts(string[] categories, int labelIndex)
        {
            SwitchParts(categories, labels[labelIndex]);
        }

        public void SwitchAllParts(string label)
        {
            SwitchParts(spriteResolverByCategory.Keys.ToArray(), label);

            OnAllPartsSwitched?.Invoke(label, Array.IndexOf(labels, label));
        }

        private void Start()
        {
            InitializeLabels();
            InitializeResolverCategories();

            SwitchAllParts(labels[0]);
        }

        private void InitializeResolverCategories()
        {
            spriteResolverByCategory = new Dictionary<string, SpriteResolver>();

            foreach (var spriteResolver in spriteResolvers)
            {
                spriteResolverByCategory[spriteResolver.GetCategory()] = spriteResolver;
            }
        }

        private void InitializeLabels()
        {
            var spriteResolver = spriteResolvers[0];
            SpriteLibraryAsset library = spriteResolver.spriteLibrary.spriteLibraryAsset;
            var categoryName = spriteResolver.GetCategory();

            labels = library.GetCategoryLabelNames(categoryName).ToArray();
        }
    }
}